package gregapi.render;

import net.minecraft.world.IBlockAccess;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public interface IRenderedBlock {
	/** @return the Textures rendered by {@link RendererBlockTextured} for the Inventory Rendering */
    @SideOnly(Side.CLIENT)
	public ITexture getTexture(int aRenderPass, byte aSide, short aMetaData);
	
	/** @return the Textures rendered by {@link RendererBlockTextured} for the World Rendering */
    @SideOnly(Side.CLIENT)
	public ITexture getTexture(int aRenderPass, byte aSide, IBlockAccess aWorld, int aX, int aY, int aZ);
	
	/** sets the Block Size rendered by {@link RendererBlockTextured} for the Inventory Rendering. return false for letting it select the normal Block Bounds. */
    @SideOnly(Side.CLIENT)
	public boolean setBlockBounds(int aRenderPass, byte aSide, short aMetaData);
	
	/** sets the Block Size rendered by {@link RendererBlockTextured} for the World Rendering. return false for letting it select the normal Block Bounds. */
    @SideOnly(Side.CLIENT)
	public boolean setBlockBounds(int aRenderPass, byte aSide, IBlockAccess aWorld, int aX, int aY, int aZ);
    
    /** gets the Amount of Render Passes for this Block for the Inventory Rendering */
    @SideOnly(Side.CLIENT)
    public int getRenderPasses(byte aSide, short aMetaData);
    
    /** gets the Amount of Render Passes for this Block for the World Rendering */
    @SideOnly(Side.CLIENT)
    public int getRenderPasses(byte aSide, IBlockAccess aWorld, int aX, int aY, int aZ);
    
    /** if this Block lets the TileEntity or a similar Handler do all the Inventory Render work. */
    @SideOnly(Side.CLIENT)
    public IRenderedBlockObject passRenderingToObject(short aMetaData);
    
    /** if this Block lets the TileEntity or a similar Handler do all the World Render work. */
    @SideOnly(Side.CLIENT)
    public IRenderedBlockObject passRenderingToObject(IBlockAccess aWorld, int aX, int aY, int aZ);
}