package gregapi.tileentity;

import static gregapi.data.CS.NW_API;
import gregapi.log.GT_Log;
import gregapi.network.INetworkHandler;
import gregapi.network.IPacket;

/**
 * @author Gregorius Techneticies
 * 
 * TileEntity with Network Code
 */
public abstract class TileEntityBase3 extends TileEntityBase2 {
	/** Variable for seeing if the Tick Function is called right now. */
	public boolean mIsRunningTick = false;
	
	/** Variable for updating Data to the Client */
	private boolean mSendClientData = false;
	
	/** @return a Packet containing all Data which has to be synchronised to the Client */
	public abstract IPacket getClientDataPacket(boolean aSendAll);
	
	/** sends all Data to the Clients in Range */
	public void sendClientData(boolean aSendAll) {getNetworkHandler().sendToAllPlayersInRange(getClientDataPacket(aSendAll), worldObj, xCoord, zCoord);}
	
	/** @return the used Network Handler. Defaults to the API Handler. */
	public INetworkHandler getNetworkHandler() {return NW_API;}
	
	/** Called to send all Data to the close Clients */
	public void updateClientData() {mSendClientData = true;}
	
	@Override public void onCoordinateChange() {super.onCoordinateChange(); updateClientData();}
	
	@Override public net.minecraft.network.Packet getDescriptionPacket() {updateClientData(); return null;}
    
    @Override
    public final void updateEntity() {
    	mIsRunningTick = true;
    	try {
	    	boolean tIsServerSide = isServerSide();
	    	if (mTimer == 0) {
	    		markDirty();
	    		onTickFirst(tIsServerSide);
	    	}
	    	onTickStart(mTimer, tIsServerSide);
	    	super.updateEntity();
	    	onTick(mTimer, tIsServerSide);
	    	if (tIsServerSide && (mSendClientData || onTickCheck(mTimer, tIsServerSide))) {
	    		sendClientData(mSendClientData);
	    		mSendClientData = false;
	    	}
	    	onTickResetChecks(mTimer, tIsServerSide);
	    	onTickEnd(mTimer, tIsServerSide);
    	} catch(Throwable e) {
    		e.printStackTrace(GT_Log.err);
    	}
    	mIsRunningTick = false;
    }
    
    /** Used to reset all Variables which have something to do with the detection of Changes. A super Call is important for this one! */
    public void onTickResetChecks(long aTimer, boolean aIsServerSide) {/**/}
    
    /** The very first Tick happening to this TileEntity */
    public void onTickFirst(boolean aIsServerSide) {/**/}
    
    /** The first Part of the Tick. */
    public void onTickStart(long aTimer, boolean aIsServerSide) {/**/}
    
    /** The regular Tick. */
    public void onTick(long aTimer, boolean aIsServerSide) {/**/}
    
    /** Use this to check if it is required to send an update to the Clients. */
    public boolean onTickCheck(long aTimer, boolean aIsServerSide) {return false;}
    
    /** The absolutely last Part of the Tick. */
    public void onTickEnd(long aTimer, boolean aIsServerSide) {/**/}
}