package gregapi.tileentity;

import static gregapi.data.CS.NBT_MTE_ID;
import static gregapi.data.CS.NBT_MTE_REG;
import static gregapi.data.CS.W;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetDrops;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetPickBlock;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnRegistrationFirst;
import gregapi.block.multitileentity.*;
import gregapi.code.ArrayListNoNulls;
import gregapi.network.IPacket;
import gregapi.network.packets.ids.PacketSyncDataIDs;
import gregapi.render.IRenderedBlockObject;
import gregapi.util.UT;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.MovingObjectPosition;
import cpw.mods.fml.common.registry.GameRegistry;

/**
 * @author Gregorius Techneticies
 */
public abstract class TileEntityBase4 extends TileEntityBase3 implements IRenderedBlockObject, IMTE_GetPickBlock, IMTE_OnRegistrationFirst, IMTE_GetDrops {
	private short mMTEID = W, mMTERegistry = W;
	
	/** return the internal Name of this TileEntity to be registered. DO NOT START YOUR NAME WITH "gt."!!! */
	public abstract String getTileEntityName();
	public abstract boolean canDrop(int aInventorySlot);
	
	@Override
	public void onRegistrationFirst(MultiTileEntityRegistry aRegistry) {
		GameRegistry.registerTileEntity(getClass(), getTileEntityName());
	}
	
	@Override
	public IPacket getClientDataPacket(boolean aSendAll) {
		return aSendAll ? new PacketSyncDataIDs(xCoord, yCoord, zCoord, getMultiTileEntityRegistryID(), getMultiTileEntityID()) : null;
	}
	
	@Override
	public final void initFromNBT(NBTTagCompound aNBT, short aMTEID, short aMTERegistry) {
		// Set ID and Registry ID.
		mMTEID = aMTEID;
		mMTERegistry = aMTERegistry;
		// Read the Default Parameters from NBT.
		if (aNBT != null) readFromNBT(aNBT);
	}
	
    @Override
    public final void readFromNBT(NBTTagCompound aNBT) {
    	// Check if this is a World/Chunk Loading Process calling readFromNBT.
        if (mMTEID == W || mMTERegistry == W) {
        	// Yes it is, so read the ID Tags first.
	        mMTEID = aNBT.getShort(NBT_MTE_ID);
	        mMTERegistry = aNBT.getShort(NBT_MTE_REG);
	        // And add additional Default Parameters, in case the Mod updated with new ones.
	        MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry(mMTERegistry);
	        if (tRegistry != null) {
	        	MultiTileEntityClassContainer tClass = tRegistry.getClassContainer(mMTEID);
	        	if (tClass != null) {
	        		// Add the Default Parameters.
	        		aNBT = UT.NBT.fuseNBTs(aNBT, tClass.mParameters);
	        	}
	        }
        }
        // now call the super Function to read XYZ.
        super.readFromNBT(aNBT);
        // And now your custom readFromNBT.
        readFromNBT2(aNBT);
    }
    
    public void readFromNBT2(NBTTagCompound aNBT) {/**/}
    
    @Override
    public final void writeToNBT(NBTTagCompound aNBT) {
        super.writeToNBT(aNBT);
        aNBT.setShort(NBT_MTE_ID, mMTEID);
        aNBT.setShort(NBT_MTE_REG, mMTERegistry);
        writeToNBT2(aNBT);
    }
    
    public void writeToNBT2(NBTTagCompound aNBT) {/**/}
    
	@Override public ArrayListNoNulls<ItemStack> getDrops(int aFortune, boolean aSilkTouch) {ArrayListNoNulls<ItemStack> rList = new ArrayListNoNulls<ItemStack>(); MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry(mMTERegistry); if (tRegistry != null) rList.add(tRegistry.getItem(mMTEID)); return rList;}
	@Override public ItemStack getPickBlock(MovingObjectPosition aTarget) {MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry(mMTERegistry); return tRegistry == null ? null : tRegistry.getItem(mMTEID);}
	@Override public short getMultiTileEntityID() {return mMTEID;}
	@Override public short getMultiTileEntityRegistryID() {return mMTERegistry;}
    @Override public void setShouldRefresh(boolean aShouldRefresh) {mShouldRefresh = aShouldRefresh;}
}